@@ -110,6 +110,10 @@ class RecordingGenerator:
                 self.spike_traces = rec_dict['spike_traces']
             else:
                 self.spike_traces = np.array([])
+            if 'template_ids' in rec_dict.keys():
+                self.template_ids = rec_dict['template_ids']
+            else:
+                self.template_ids = None
             self.info = deepcopy(info)
             self.params = deepcopy(info)
             if len(self.spiketrains) > 0:
@@ -136,11 +140,13 @@ class RecordingGenerator:
             self.spgen = spgen
             self.tempgen = tempgen
             self.tmp_mode = None
+            self.template_ids = None
 
         self.overlapping = []
         # temp file that should remove on delete
         self._to_remove_on_delete = []
         self.tmp_folder = None
+        self.n_jobs = None
         self._is_tmp_folder_local = False
 
     def __del__(self):
@@ -168,7 +174,7 @@ class RecordingGenerator:
         self.recordings = None
         self.spike_traces = None
 
-    def generate_recordings(self, tmp_mode=None, tmp_folder=None, verbose=None, n_jobs=0):
+    def generate_recordings(self, tmp_mode=None, tmp_folder=None, verbose=None, template_ids=None, n_jobs=0):
         """
         Generates recordings
 
@@ -180,13 +186,18 @@ class RecordingGenerator:
         tmp_folder: str or Path
             In case of tmp files, you can specify the folder.
             If None, then it is automatic using tempfile.mkdtemp()
+        n_jobs: int
+            if >1 then use joblib to execute chunk in parallel else in loop
+        template_ids: list or None
+            If None, templates are selected randomly based on selection rules. If a list of indices is provided, the
+            indices are used to select templates (template selection is bypassed)
         verbose: bool or int
             Determines the level of verbose. If 1 or True, low-level, if 2 high level, if False, not verbose
-        n_jobs: int if >1 then use joblib to execute chunk in parallel else in loop
         """
 
         self.tmp_mode = tmp_mode
         self.tmp_folder = tmp_folder
+        self.template_ids = template_ids
         self.n_jobs = n_jobs
 
         if tmp_mode is not None:
@@ -651,28 +662,39 @@ class RecordingGenerator:
 
                 if verbose_1:
                     print('Selecting cells')
-                idxs_cells, selected_cat = select_templates(locs, eaps, bin_cat, n_exc, n_inh, x_lim=x_lim, y_lim=y_lim,
-                                                            z_lim=z_lim, min_amp=min_amp, max_amp=max_amp,
-                                                            min_dist=min_dist, drifting=drifting,
-                                                            drift_dir=drift_directions,
-                                                            preferred_dir=preferred_dir, angle_tol=angle_tol,
-                                                            n_overlap_pairs=n_overlap_pairs,
-                                                            overlap_threshold=overlap_threshold,
-                                                            verbose=verbose_2)
 
-                if not np.any('U' in  selected_cat):
-                    assert selected_cat.count('E') == n_exc and selected_cat.count('I') == n_inh
-                    # Reorder templates according to E-I types
-                    reordered_idx_cells = np.array(idxs_cells)[np.argsort(selected_cat)]
+                if self.template_ids is None:
+                    idxs_cells, selected_cat = select_templates(locs, eaps, bin_cat, n_exc, n_inh, x_lim=x_lim,
+                                                                y_lim=y_lim, z_lim=z_lim, min_amp=min_amp,
+                                                                max_amp=max_amp, min_dist=min_dist, drifting=drifting,
+                                                                drift_dir=drift_directions,
+                                                                preferred_dir=preferred_dir, angle_tol=angle_tol,
+                                                                n_overlap_pairs=n_overlap_pairs,
+                                                                overlap_threshold=overlap_threshold,
+                                                                verbose=verbose_2)
+
+                    if not np.any('U' in  selected_cat):
+                        assert selected_cat.count('E') == n_exc and selected_cat.count('I') == n_inh
+                        # Reorder templates according to E-I types
+                        reordered_idx_cells = np.array(idxs_cells)[np.argsort(selected_cat)]
+                    else:
+                        reordered_idx_cells = idxs_cells
+
+                    template_celltypes = celltypes[reordered_idx_cells]
+                    template_locs = np.array(locs)[reordered_idx_cells]
+                    template_rots = np.array(rots)[reordered_idx_cells]
+                    template_bin = np.array(bin_cat)[reordered_idx_cells]
+                    templates = np.array(eaps)[reordered_idx_cells]
+                    self.original_templates = templates
+                    self.template_ids = reordered_idx_cells
                 else:
-                    reordered_idx_cells = idxs_cells
-
-                template_celltypes = celltypes[reordered_idx_cells]
-                template_locs = np.array(locs)[reordered_idx_cells]
-                template_rots = np.array(rots)[reordered_idx_cells]
-                template_bin = np.array(bin_cat)[reordered_idx_cells]
-                templates = np.array(eaps)[reordered_idx_cells]
-                self.original_templates = templates
+                    print(f"Using provided template ids: {self.template_ids}")
+                    template_celltypes = celltypes[self.template_ids]
+                    template_locs = np.array(locs)[self.template_ids]
+                    template_rots = np.array(rots)[self.template_ids]
+                    template_bin = np.array(bin_cat)[self.template_ids]
+                    templates = np.array(eaps)[self.template_ids]
+                    self.original_templates = templates
 
                 # find overlapping templates
                 overlapping = find_overlapping_templates(templates, thresh=overlap_threshold)
